﻿//
// Created by zyz on 2024/9/30.
//

#include "PerformanceTool.h"
#include "../Utils/StringUtil.h"
#include <Psapi.h>
#include <TlHelp32.h>
#include <Windows.h>
#include <powersetting.h>
#include <tchar.h>
#include <numeric>
#include <vector>
#include <thread>
#include <string>
#include <windows.h>
#include <psapi.h>
#include <string>
#include <tchar.h>
#include <iostream>


#pragma comment(lib, "PowrProf.lib")
#pragma comment(lib, "pdh.lib")

WinDevice::PerformanceTool::PerformanceTool() {

}

WinDevice::PerformanceTool::~PerformanceTool() {

}

float WinDevice::PerformanceTool::GetAppCpuUsage(DWORD processId) {
    return _GetAppCpuUsage(processId, _GetProcessName(processId));
}

float WinDevice::PerformanceTool::GetAppCpuUsage(const std::wstring &processName) {
    return _GetAppCpuUsage(_GetProcessId(processName), processName);
}

float WinDevice::PerformanceTool::_GetAppCpuUsage(DWORD processId, const std::wstring &processName) {
    HQUERY query;
    PDH_HCOUNTER cpuCounter;
    PDH_FMT_COUNTERVALUE counterVal;
    TCHAR szCounterPath[MAX_PATH];

    // 初始化PDH
    if (PdhOpenQuery(NULL, 0, &query) != ERROR_SUCCESS) {
        return 1;
    }

    // 构造CPU计数器路径
    _stprintf_s(szCounterPath, MAX_PATH, TEXT("\\Process V2(%s:%d)\\%% Processor Time"), WinDevice::Wstring2String(processName).c_str(), processId);
    if (PdhAddCounter(query, szCounterPath, 0, &cpuCounter) != ERROR_SUCCESS) {
        PdhCloseQuery(query);
        return 1;
    }

    PdhCollectQueryData(query);
    std::this_thread::sleep_for(std::chrono::milliseconds(1000)); // 等待以获取更新数据
// 再次收集数据
    PdhCollectQueryData(query);

    // 读取计数器值
    auto cpuUsage = 0.0;
    auto result = PdhGetFormattedCounterValue(cpuCounter, PDH_FMT_DOUBLE, NULL, &counterVal);
    if (result == ERROR_SUCCESS) {
        cpuUsage = counterVal.doubleValue;
    }

    // 清理
    // 关闭句柄
    PdhRemoveCounter(cpuCounter);
    PdhCloseQuery(query);
    return cpuUsage;
}

float WinDevice::PerformanceTool::GetAppMemoryUsage(const std::wstring &processName) {
    return _GetAppMemoryUsage(processName);
}

float WinDevice::PerformanceTool::GetAppMemoryUsage(DWORD processId) {
    return _GetAppMemoryUsage(_GetProcessName(processId));
}

float WinDevice::PerformanceTool::_GetAppMemoryUsage(const std::wstring &processName) {
    HQUERY query;
    PDH_HCOUNTER memCounter;
    PDH_FMT_COUNTERVALUE counterVal;
    TCHAR szCounterPath[MAX_PATH];

    // 初始化PDH
    if (PdhOpenQuery(NULL, 0, &query) != ERROR_SUCCESS) {
        return 1;
    }

    // 构造内存计数器路径
    _stprintf_s(szCounterPath, MAX_PATH, TEXT("\\Process(%s)\\%s"), processName.c_str(), TEXT("Working Set - Private"));
    if (PdhAddCounter(query, szCounterPath, 0, &memCounter) != ERROR_SUCCESS) {
        PdhCloseQuery(query);
        return 1;
    }

    PdhCollectQueryData(query);

    auto memoryUsage = 0.0;
    auto result = PdhGetFormattedCounterValue(memCounter, PDH_FMT_DOUBLE, NULL, &counterVal);
    if (result == ERROR_SUCCESS) {
        memoryUsage = counterVal.doubleValue / 1024.0 / 1024.0;
    }

    // 清理
    // 关闭句柄
    PdhRemoveCounter(memCounter);
    PdhCloseQuery(query);
    return memoryUsage;
}

std::wstring WinDevice::PerformanceTool::_GetProcessName(DWORD processId) {
    std::wstring processName = L"<unknown>"; // 使用 std::wstring
    HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, processId);
    if (hProcess != nullptr) {
        HMODULE hMod;
        DWORD cbNeeded;

        // 获取进程中的模块句柄
        if (EnumProcessModules(hProcess, &hMod, sizeof(hMod), &cbNeeded)) {
            // 获取模块的可执行文件名
            TCHAR name[MAX_PATH];
            GetModuleBaseName(hProcess, hMod, name, sizeof(name) / sizeof(TCHAR));

            // 去掉 .exe 后缀
            TCHAR* dot = _tcsrchr(name, '.');
            if (dot != nullptr && _tcscmp(dot, TEXT(".exe")) == 0) {
                *dot = '\0'; // 将 '.' 及其后面的字符替换为 '\0'
            }

            processName = WinDevice::TCHARArrayToWString(name); // 转换为 std::wstring
        }
        CloseHandle(hProcess); // 确保在这里关闭句柄
    }

    // 打印进程名称
    std::wcout << L"Current process name: " << processName << std::endl;
    return processName; // 返回 std::wstring
}

DWORD WinDevice::PerformanceTool::_GetProcessId(const std::wstring &processName) {
    DWORD processId = 0;
    //创建进程快照
    HANDLE hSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    if (hSnapshot == INVALID_HANDLE_VALUE)
    {
        return 0;
    }

    PROCESSENTRY32 pe;
    pe.dwSize = sizeof(PROCESSENTRY32);

// 遍历所有进程
    if (Process32First(hSnapshot, &pe))
    {
        do
        {
            // 将进程名与目标进程名进行比较
            if (_tcscmp(pe.szExeFile, WinDevice::Wstring2String(processName).c_str()) == 0)
            {
                processId = pe.th32ProcessID;
                break;
            }
        } while (Process32Next(hSnapshot, &pe));
    }
    return processId;
}
