﻿//
// Created by zyz on 2024/9/30.
//

#ifndef WINDEVICE_STRINGUTIL_CPP_H
#define WINDEVICE_STRINGUTIL_CPP_H

#include "StringUtil.h"
#include <windows.h>
#include <string>
#include <locale>
#include <codecvt>

template<typename To, typename From>
To Convert(const From &input) {
    std::wstring_convert<std::codecvt_utf8<typename From::value_type>, typename From::value_type> converter;
    try {
        return converter.to_bytes(input);
    } catch (...) {
        return To(); // 返回空对象
    }
}

std::string WinDevice::Wstring2String(const std::wstring &input) {
    return Convert<std::string>(input);
}

std::string WinDevice::Wchar2String(const WCHAR *input) {
    std::wstring_convert<std::codecvt_utf8<wchar_t>> converter;
    try {
        return converter.to_bytes(input);
    } catch (...) {
        return ""; // 返回空字符串表示失败
    }
}

std::wstring WinDevice::String2Wstring(const std::string &input) {
    std::wstring_convert<std::codecvt_utf8<wchar_t>> converter;
    try {
        return converter.from_bytes(input);
    } catch (...) {
        return L""; // 返回空宽字符串表示失败
    }
}

wchar_t *WinDevice::String2Wchar(const std::string &input) {
    std::wstring wstr = String2Wstring(input);
    if (wstr.empty()) {
        return nullptr; // 转换失败时返回 nullptr
    }
    wchar_t *wcharStr = new wchar_t[wstr.length() + 1];
    std::wcscpy(wcharStr, wstr.c_str());
    return wcharStr; // 返回动态分配的 wchar_t*
}

// 通用转换函数
std::wstring WinDevice::TCHARArrayToWString(const TCHAR *tcharArray) {
#ifdef UNICODE
    return std::wstring(tcharArray); // 如果是 Unicode，直接返回
#else
    // 如果是 ANSI，进行转换
    int size_needed = MultiByteToWideChar(CP_ACP, 0, tcharArray, -1, nullptr, 0);
    std::wstring wstrName(size_needed, L'\0'); // 创建足够大小的 std::wstring
    MultiByteToWideChar(CP_ACP, 0, tcharArray, -1, &wstrName[0], size_needed);
    return wstrName;
#endif
}


#endif //WINDEVICE_STRINGUTIL_CPP_H
